<?php
/*
 * Author: Kostyantyn Gushtin <kgushtin@n-ix.com.ua>
 *
 * Daemon responsible for joining together loadfiles and copying them to
 * storage directory.
 *
 * */

// Make sure all errors are displayed
error_reporting(E_ALL);
ini_set("display_errors", "On");

require_once("logger.inc.php");

function usage() {
	global $argv;

	print "Purge logs from local/remote archive\n";
	print "syntax: $argv[0] local|remote\n";
	print "\n";
	print " local           purge local archive\n";
	print " remote          purge remote archive\n";
	exit;
}

if (!isset($argv) || count($argv) <= 1) {
	usage();
}

switch ($argv[1]) {
	case "local":
	case "remote":
		break;
	default:
		usage();
}

Logger::setSyslog(true, "op5-cleanup");

class LogCleaner
{
	private $storagePaths = array("local" => "/opt/logserver/archive", "remote" => "");
	private $units = array("local" => "days", "remote" => "months");
	private $mode = 0750;
	private $owner = "root:postgres";

	private function setPermissions($file, $mode = 0755, $owner = 'root:root') {
		if(strpos($owner, ':')) {
			list($uid, $gid) = explode(':', $owner);
		} else {
			$gid = 'root';
			$uid = $owner;
		}

		chmod($file, $mode);
		chown($file, $uid);
		chgrp($file, $gid);
	}

	function mkdirRecursive($dir, $mode, $owner)
	{
		$old_umask = umask(0);

		if (is_dir($dir))
			return;

		Logger::debug("Creating directory $dir (and leading) with mode %o\n", $mode);

		$dir_parts = explode('/', $dir);
		# add one extra array item so we can shift them all out
		$dir_parts[] = '';
		$fd = array_shift($dir_parts);
		foreach ($dir_parts as $dp) {
			if ($fd) {
				if (!is_dir($fd)) {
					Logger::debug("Creating directory '%s' with mode %o and group %s\n", $fd, $mode, $owner);
					mkdir($fd);
                    $this->setPermissions($fd, $mode, $owner);
				}
			}

			$fd .= "/$dp";
		}

		umask($old_umask);
	}

	function daemonMain()
	{
		global $argv;

		if (pg_connect("dbname=op5lsu user=op5lsu"))
		{
			$this->loadOptions();
			$this->cleanOldLogs($argv[1]);
			pg_close();
		}
		else
		{
			Logger::log("Please, start PostgreSQL.");
		}

	}

	function loadOptions()
	{
		$query = "SELECT local_storage_path, remote_storage_path FROM settings";
		Logger::debug("Trying to execute pg_query(%s)", $query);

		$retval = true;
		$query_result = pg_query($query);

		if (!$query_result) {
			Logger::log("Got error from pgsql: %s", pg_last_error());
			$retval = false;
		}
		else
		{

			$paths = pg_fetch_row($query_result);

			if (!empty($paths[0]))
				$this->storagePaths["local"] = $paths[0];

			if (!empty($paths[1]))
				$this->storagePaths["remote"] = $paths[1];
		}

		return $retval;
	}

	function cleanOldLogs($type)
	{
		$query = "SELECT tables_registry.name FROM tables_registry, settings"
			. " WHERE ((location & get_location_mask( '$type' ))::integer != 0)"
			. " AND (date + (keep_in_$type || ' {$this->units[$type]}')::interval < current_timestamp)"
			. " AND ((import_status = 'finished') OR (import_status = 'failed') OR import_status IS NULL)";
		Logger::debug("Trying to execute pg_query(%s)", $query);

		$retval = true;
		$query_result = pg_query($query);

		if (!$query_result) {
			Logger::log("Got error from pgsql: %s", pg_last_error());
			$retval = false;
		}
		else
		{
			while ($row = pg_fetch_row($query_result))
			{
				$log_time = explode("_", substr($row[0], 9));
				$name = $this->storagePaths[$type]."/".$log_time[0]."_".$log_time[1]."/".$log_time[2]."/".$log_time[3].".bz2";

				if ($type == 'local' && $this->storagePaths['remote'])
				{
					$result = pg_query("SELECT (location & get_location_mask('remote'))::integer AS has_remote"
						. " FROM tables_registry WHERE name = '{$row[0]}'");
					$has_remote = pg_fetch_row($result);

					$dir2 = $this->storagePaths['remote']."/".$log_time[0]."_".$log_time[1]."/".$log_time[2];
					$name2 = $dir2."/".$log_time[3].".bz2";

					if (!$has_remote[0] || !file_exists($name2)) # if not tagged in db as remote or if file does not exist in remote storage
					{
						$this->mkdirRecursive($dir2, $this->mode, $this->owner);

						$cmd = "mv ".$name.' '.$name2;
						exec($cmd, $output, $retval);
						if ($retval)
							Logger::log("Can't move log $name to $name2 !");
						else
						{
							# add remote storage tag to table
							pg_query("UPDATE tables_registry SET location = (location | get_location_mask('remote')) WHERE name = '{$row[0]}'");
							# remove local storage tag from table
							$this->unregisterLog($row[0], $type);
						}
					}
					else
					{
						# remove log file from disk
						if ($this->eraseLog($name))
							# remove $type storage tag from table
							$this->unregisterLog($row[0], $type);
					}
				}
				else
					# remove log file from disk
					if ($this->eraseLog($name))
						# remove $type storage tag from table
						$this->unregisterLog($row[0], $type);

				unset($log_time);
			}
		}

		return $retval;
	}

	function eraseLog($name)
	{
		$cmd = "rm ".$name;
		exec($cmd, $output, $retval);
		if ($retval)
		{
			Logger::log("Can't delete old log $name !");
			return false;
		}

		return true;
	}

	function unregisterLog($name, $storage)
	{
		$query = "SELECT unregister_table('$name', get_location_mask('$storage'))";
		$retval = true;

		if (!pg_query($query))
		{
			Logger::log("Can't unregister $storage log ! pgsql: %s", pg_last_error());
			$retval = false;
		}

		return $retval;
	}
}

$lc = new LogCleaner();
$lc->daemonMain();

exit;

?>
