<?php

/*
 * Author: Kostyantyn Gushtin <kgushtin@n-ix.com.ua>
 * Author: Peter Ostlin <peter@op5.se>
 *
 * Daemon responsible for joining together loadfiles and copying them to
 * storage directory.
 *
 * */

// Make sure all errors are displayed
error_reporting(E_ALL);
ini_set("display_errors", "On");

require_once("logger.inc.php");
require_once("daemon.inc.php");

declare(ticks = 1);

class RotateDaemon extends Daemon
{

	private $dirmode = 0751;
	private $filemode = 0640;
	private $owner = "root:postgres";
	private $archivePath = "/var/log/oslogd/archive"; // Default archive path
	private $localStoragePath = "/opt/logserver/archive"; // Default local archive path
	private $remoteStoragePath = ""; // Default remote archive path
	
	private function setPermissions($file, $mode = 0750, $owner = 'root:root') {
		if(strpos($owner, ':')) {
			list($uid, $gid) = explode(':', $owner);
		} else {
			$gid = 'root';
			$uid = $owner;
		}

		chmod($file, $mode);
		chown($file, $uid);
		chgrp($file, $gid);
	}

	function __construct()
	{
		parent::__construct("op5-rotate", "op5-rotate");
	}

	function daemonMain()
	{

		if (!pg_connect("dbname=op5lsu user=op5lsu"))
		{
			Logger::log("Please, start PostgreSQL.");
		}

		$this->loadOptions();
		$this->rotateFiles();

		pg_close();
	}

	function parseCmdArgument($parameter, $parameterArray)
	{

		$this->archivePath = $parameter;
		return true;
	}

	function loadOptions()
	{

		$query = "select local_storage_path,remote_storage_path from settings;";
		Logger::debug("Trying to execute pg_query(%s)", $query);

		$retval = true;
		$query_result = pg_query($query);

		if (!$query_result) {
			Logger::log("Got error from pgsql: %s", pg_last_error());
			$retval = false;
		}
		else
		{

			$paths = pg_fetch_row($query_result);

			if (!empty($paths[0]))
				$this->localStoragePath = $paths[0];

			if (!empty($paths[1]))
				$this->remoteStoragePath = $paths[1];
		}

		return $retval;

	}

	function testArgValidity($file)
	{

		$ret = false;
		if (is_file($file) && !is_link($file))
			if (is_readable($file) && is_writable($file))
			{
				$filename = basename($file);
				if (ereg("([0-9]{4})_([0-9]{2})_([0-9]{2})_([0-9]{2})_([0-9]{2})", $filename))
				{
					$dt = explode("_", substr($filename, 4));
					$tm = strtotime($dt[0] . "-" . $dt[1] . "-" . $dt[2] . " " . $dt[3] . ":" . $dt[4] . ":00");

					if ($tm != -1 && $tm != "")
						$ret = true;
				}
			}

		return $ret;

	}

	function rotateFiles()
	{

		$dirfiles = glob($this->archivePath . "/cfs.*");

		$logfiles = array();
		foreach ($dirfiles as $filename)
		{
			if ($this->testArgValidity($filename))
				$logfiles[] = $filename;
		}

		sort($logfiles);

		$log_time = explode("_", substr(basename($logfiles[0]), 4));
		$old_hour = $log_time[3];						// LOG HOUR
		$old_date_part = $log_time[0]."_".$log_time[1]."/".$log_time[2];	// LOG DATE
		unset($log_time);

		foreach($logfiles as $one_file) {

			$log_time = explode("_", substr(basename($one_file), 4));
			$cur_hour = $log_time[3];
			$date_part = $log_time[0]."_".$log_time[1]."/".$log_time[2];
			unset($log_time);

			if ($old_hour != $cur_hour || $date_part != $old_date_part) {

				$local_path = $this->localStoragePath."/".$old_date_part."/";
				if (empty($this->remoteStoragePath))
					$remote_path = "";
				else
					$remote_path = $this->remoteStoragePath."/".$old_date_part."/";

				$this->compressFiles($hour_files, $local_path, $remote_path, $old_hour.".bz2");

				$old_hour = $cur_hour;
				$old_date_part = $date_part;
				unset($hour_files);
			}

			$hour_files[] = $one_file;

		}

		$cur_date = date("Y_m/d_H");
		$old_date = $old_date_part . "_". $old_hour;
		if ($old_date != $cur_date && !empty($hour_files)) {

				$local_path = $this->localStoragePath."/".$old_date_part."/";
				if (empty($this->remoteStoragePath))
					$remote_path = "";
				else
					$remote_path = $this->remoteStoragePath."/".$old_date_part."/";

				$this->compressFiles($hour_files, $local_path, $remote_path, $old_hour.".bz2");

		}

		unset($hour_files);
		unset($logfiles);

	}

	function mkdirRecursive($dir, $mode, $owner)
	{
		$old_umask = umask(0);

		if (is_dir($dir))
			return;

		Logger::debug("Creating directory $dir (and leading) with mode %o\n", $mode);

		$dir_parts = explode('/', $dir);
		# add one extra array item so we can shift them all out
		$dir_parts[] = '';
		$fd = array_shift($dir_parts);
		foreach ($dir_parts as $dp) {
			if ($fd) {
				if (!is_dir($fd)) {
					Logger::debug("Creating directory '%s' with mode %o and group %s\n", $fd, $mode, $owner);
					mkdir($fd);
					$this->setPermissions($fd, $mode, $owner);
				}
			}

			$fd .= "/$dp";
		}

		umask($old_umask);
	}

	function compressFiles($file_ary, $local_dir, $remote_dir, $outfile)
	{
		$this->mkdirRecursive($local_dir, $this->dirmode, $this->owner);
		$file_list = "'".implode("' '", $file_ary)."'";
		$cmd = "cat $file_list | pbzip2 >> ".$local_dir.$outfile;
		exec($cmd, $output, $retval);
		$this->setPermissions($local_dir.$outfile, $this->filemode, $this->owner);
		if ($retval) {
			Logger::log("Concatenation / compression failed. Aborting delete and remote saving");
			return false;
		}

		$file_name = basename($file_ary[0]);
		$name = "messages_".substr($file_name, 4, strlen($file_name) - 7);
		$this->registerLog($name, "local");

		$this->mkdirRecursive($remote_dir, $this->dirmode, $this->owner);
		if (!empty($remote_dir) && file_exists($remote_dir))
		{
			$cmd = "cp ".$local_dir.$outfile." ".$remote_dir.$outfile;
			exec($cmd, $output, $retval);
			$this->setPermissions($remote_dir.$outfile, $this->filemode, $this->owner);
			$this->registerLog($name, "remote");
		}

		$cmd = "rm $file_list";
		exec($cmd, $output, $retval);

		return true;
	}

	function registerLog($name, $storage)
	{

		$query = "select register_table('$name', get_location_mask('$storage'));";
		$retval = true;

		if (!pg_query($query))
		{
			Logger::log("Can't register $storage log ! pgsql: %s", pg_last_error());
			$retval = false;
		}

		return $retval;

	}

}

$rd = new RotateDaemon();
$rd->run();

exit; // Main program done, functions below

?>
